﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Bill.Settlement.Selector', {
    extend: 'System.Control.EntitySelector',
    entityType: 'Kivii.Finances.Entities.Settlement',
    xtype: 'App.Control.Kivii.Finances.Bill.Settlement.Selector',
    dataSource: '/Restful/Kivii.Finances.Entities.Settlement/QueryEx.json',
    filter: {
        IsPayed: false,
        IsBilled: false,
        OwnerKvid:KiviiContext.CurrentMember.DepartmentKvid
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    store: {
        autoLoad: true,
        pageSize: 50,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    tbar: [{
        xtype: 'combo',
        queryMode: 'local',
        displayField: 'displayInfo',
        valueField: 'isBilled',
        value: false,
        editable: false,
        filterPickList: true,
        reference: 'isBilled',
        store: {
            fields: ['displayInfo', 'isBilled'],
            data: [{
                displayInfo: '全部',
                isBilled: null
            }, {
                displayInfo: '已做账单',
                isBilled: true
            }, {
                displayInfo: '未做账单',
                isBilled: false
            }]
        },
        listeners: {
            select: '_onSelectIsBilled'
        }
    },{
        xtype: 'datefield',
        fieldLabel: '时间范围  开始',
        format: 'Y-m-d',
        labelWidth: 90,
        reference: 'startTime',
        editable: false,
        triggers: {
            clear: {
                weight: 2,
                cls: 'x-fa fa-times',
                handler: '_cleanBeginOperateTime'
            }
        },
        listeners: {
            select: '_onSelectMonth'
        }
    },
    {
        xtype: 'datefield',
        fieldLabel: '结束',
        format: 'Y-m-d',
        labelWidth: 30,
        reference: 'endTime',
        editable: false,
        triggers: {
            clear: {
                weight: 2,
                cls: 'x-fa fa-times',
                handler: '_cleanEndOperateTime'
            }
        },
        listeners: {
            select: '_onSelectMonth'
        }
    }, {
        xtype: 'searchField',
        flex: 1,
        labelWidth: 60,
        queryField: 'PayerName,BizId,OperatorName',
        emptyText: '模糊搜索付款单位、所属业务或操作人',
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            xtype: 'rownumberer'
        }, {
            dataIndex: 'BizId',
            text: '所属业务',
            width: 170,
            // sortable: true,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('合计');
            }
        }, {
            dataIndex: 'OwnerName',
            text: '所属部门',
            width: 120
        }, {
            dataIndex: 'PayerName',
            text: '付款单位',
            width: 120
        }, {
            dataIndex: 'OperateTime',
            text: '操作日期',
            width: 100,
            // sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'AmountPlan',
            align: 'right',
            text: '原金额',
            width: 100,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'Amount',
            align: 'right',
            text: '结算金额',
            width: 100,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'AmountPayment',
            align: 'right',
            text: '已付金额',
            width: 100,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'Type',
            text: '类型',
            width: 90,
            renderer: '_renderType'
        }, {
            dataIndex: 'OperatorName',
            text: '操作人',
            width: 90
        }, {
            dataIndex: 'Summary',
            text: '摘要',
            width: 200
        }, {
            dataIndex: 'Remark',
            text: '备注',
            minWidth: 100,
            flex: 1
        }]
    },
    controller: {
    	_renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Report") {
                return "报告费";
            }
            if (_value === "Service") {
                return "服务费";
            }
            if (_value === "Express") {
                return "快递费";
            }
            return _value;
        },
        _onSelectIsBilled: function(_ctrl, _record) {
            let _controller = this;
            let _view = _controller.getView();
            _view.filter.IsBilled = _record.data.isBilled;
            _view.reload();
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            delete _view.filter.OperateTimeLessThan;
            _view.filter.OperateTimeGreaterThanOrEqualTo = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.OperateTimeLessThan = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _cleanBeginOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            _view.reload();
        },
        _cleanEndOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeLessThan;
            _view.reload();
        }
    }
});