﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Payment.Filter.Form', {
    extend: 'System.Control.EntityForm',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.Control.Kivii.Finances.Payment.Filter.Form',
    autoScroll: true,
    layout: {
        type: 'responsivecolumn',
        states: {
            small: 800,
            large: 0
        }
    },
    viewModel: {
        data: {
            Type: null,
            Amount: 0,
            // departCode: null,
            owners: [],
            Kvids: [],
            Kvid: null,
            ButtonText: null,
            ButtonIcon: null
        },
        formulas: {
            amountChineseBig: function(get) {
                let _amount = get('Amount');
                return KiviiContext.Const.FinancesFunctionsMoneyToChineseBig(_amount);
            }
        }
    },
    tbar: ['->', {
        iconAlign: 'left',
        handler: '_onSaveEntity',
        bind: {
            text: '{ButtonText}',
            iconCls: '{ButtonIcon}',
            disabled: '{!formIsValid}',
            hidden: '{!ButtonText}'
        }
    }],
    defaults: {
        margin: '0 10 10 10',
        labelWidth: 60,
        labelAlign: 'right',
        xtype: 'textfield'
    },
    items: [{
        xtype: 'numberfield',
        fieldLabel: '认领金额',
        emptyText: '请输入数字',
        hideTrigger: true,
        responsiveCls: 'big-100 small-100',
        reference: 'amount',
        minValue: 0,
        decimalPrecision: 2,
        bind: {
            value: '{Amount}'
        },
        allowBlank: false
    }, {
        xtype: 'textfield',
        responsiveCls: 'big-100 small-100',
        fieldLabel: '金额大写',
        readOnly: true,
        bind: {
            value: '{amountChineseBig}'
        }
    }],
    setOwner: function(_entities, _ctrl) { //_type针对 Refund(退款),Split(认领),UnBiz(非业务)
        let _view = this;
        let _vm = _view.getViewModel();
        let _refs = _view.getReferences();
        let _type = _ctrl.type;
        if (_entities.length <= 0) return;
        _vm.set('ButtonText', _ctrl.text);
        _vm.set('ButtonIcon', _ctrl.iconCls);
        let _owners = [];

        if (_entities.length > 1) {
            _vm.set('Type', 'Multi' + _type);
            _refs.amount.setReadOnly(true);
            let _kvids = [];
            let _amount = 0;
            for (let _i = 0; _i < _entities.length; _i++) {
                let _item = _entities[_i].data;
                _kvids.push(_item.Kvid);
                _amount += parseFloat(parseFloat(_item.Amount - _item.AmountSplited).toFixed(2));
                _owners.push(_item);
            }
            _vm.set('Amount', _amount);
            _vm.set('Kvids', _kvids);
        } else {
            _vm.set('Type', _type);
            let _item = _entities[0].data;
            let _amount = parseFloat(parseFloat(_item.Amount - _item.AmountSplited).toFixed(2));
            _refs.amount.setMaxValue(_amount);
            _vm.set('Amount', _amount);
            _vm.set('Kvid', _item.Kvid);
            _owners.push(_item);
        }
        // for (let _i = 0; _i < _owners.length; _i++) {
        // 	let _item = _owners[_i];
        // 	if (_item.Metadata.hasOwnProperty("moduleId")){
        // 		_refs.depart.allowBlank=false;
        // 		_refs.depart.setHidden(false);
        // 	}
        // }
        _vm.set('owners', _owners);
    },
    controller: {
        _onSaveEntity: function(_ctrl) {
            let _controller = this;
            
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _type = _vm.data.Type;
            if (_type === null) return;
            if (_vm.data.Amount <= 0) {
                Ext.toast('无可操作金额!');
                return;
            }
            let _jsonData = {};
            if (_type.indexOf('Multi') !== -1) _jsonData.Kvids = _vm.data.Kvids;
            else {
                _jsonData.Amount = _vm.data.Amount;
                _jsonData.Kvid = _vm.data.Kvid;
            }
            if (_jsonData === {}) return;
            _view.mask('数据通讯中...');
            let _url = '/Restful/' + _view.entityType + '/' + _type + '.json';
            // return;
            Ext.Ajax.request({
                url: _url,
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    _view.unmask();
                    Ext.toast('操作成功!');
                    _view.fireEvent('entitySplited', _controller, _rtns);
                },
                failure: function(_response) {
                    Ext.toast('操作失败!');
                    _view.unmask();
                }
            });
        }
    }
});