﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Policy.List', {
    extend: 'System.Control.EntityList',
    xtype: 'App.View.Kivii.Finances.Policy.List',
    entityType: 'Kivii.Finances.Entities.Policy',
    filter: {},
    // disabled:true,
    header: true,
    layout: 'fit',
    title: '优惠政策',
    // iconCls: 'x-fa fa-cc',
    viewModel: {
        data: {
            _selection: null,
            lastOwnerKvid: null
        },
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 50,
        sorters: [{
            property: 'CreateTime',
            direction: 'DESC'
        }]
    },
    plugins: [{
        ptype: 'cellediting',
        clicksToEdit: 0
    }],
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
            xtype: 'searchField',
            flex: 1,
            minWidth: 250,
            labelWidth: 60,
            queryField: 'Name',
            emptyText: '模糊搜索公司名称'
        }, {
            text: '新增',
            iconCls: 'x-fa fa-plus',
            iconAlign: 'left',
            handler: '_onAddPolicy',
        }, {
            text: '详情',
            iconCls: 'x-fa fa-list',
            iconAlign: 'left',
            handler: '_onReadPolicy',
            bind: {
                disabled: '{!_selection}'
            }
        },{
            text: '执行记录',
            iconCls: 'x-fa fa-signal',
            iconAlign: 'left',
            handler: '_onReadPolicyRecord',
            bind: {
                disabled: '{!_selection}'
            }
        }, {
            text: '删除',
            iconCls: 'x-fa fa-trash-o',
            iconAlign: 'left',
            handler: '_onDeleteEntity',
            bind: {
                disabled: '{!_selection}'
            }
        }]
    }, {
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
        }, {
            text: '政策名称',
            dataIndex: 'Name',
            width: 230
        }, {
            text: '所属部门',
            dataIndex: 'OwnerName',
            width: 180
        }, {
            text: '政策类型',
            dataIndex: 'Type',
            width: 100,
            renderer: '_renderType'
        // }, {
        //     dataIndex: 'Amount',
        //     text: '减免金额',
        //     align: 'right',
        //     width: 120
        }, {
            text: '减免方式',
            dataIndex: 'DiscountRate',
            width: 120,
            renderer: '_renderRate'
        }, {
            dataIndex: 'EffectiveTime',
            text: '生效日期',
            sortable: true,
            width: 100,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'ExpiredTime',
            text: '失效日期',
            sortable: true,
            width: 100,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            xtype: 'actioncolumn',
            text: '目标客户',
            align: 'center',
            items: [{
                text: '目标客户',
                iconCls: 'x-fa fa-search',
                handler: '_onReadDetail'
            }],
            width: 100
        }, {
            dataIndex: 'OperatorName',
            text: '操作人',
            width: 90
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 120
        }, {
            text: '备注',
            dataIndex: 'Remark',
            width: 150,
            flex: 1
        }]
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _renderRate: function(_value, _meta, _record, _rowIndex) {
        	let _amount=_record.data.Amount;
        	if(_amount>0){
        		return "金额: "+_amount;
        	}
            let _urgent = null;
            let _urgentRate = _record.data.DiscountRate || 1;
            if (_urgentRate === 1) _urgent = '无折扣';
            else if (_amount > 0) _urgent = '无折扣';
            else _urgent = _urgentRate * 100 + '%';
            return "折扣: "+_urgent;
        },

        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === 'Original') return '普通折扣';
            if (_value === 'Additional') return '折上折';
            return _value;
        },
        _onAddPolicy: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _item = {};
            _item.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            _item.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            _item.Type = 'Original';
            _item.Amount = 0;
            _item.DiscountRate = 0.5;
            _item.EffectiveTime = new Date();
            _item.OperateTime = new Date();
            _item.Currency = "CNY";
            _item.OperatorName = KiviiContext.CurrentMember.FullName;
            _item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            let _form = Ext.create('App.Control.Kivii.Finances.Policy.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                autoScroll: true,
                width: 660,
                height: 380,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _win.show();
            _form.newEntity(_item);
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onDeleteEntity: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            _view.deleteEntity();
        },
        _onReadPolicy: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            if (!_view.selection) return;
            let _form = Ext.create('App.Control.Kivii.Finances.Policy.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                autoScroll: true,
                width: 600,
                height: 380,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _win.show();
            _form.readEntity(_view.selection.data, true);
            _form.on('entityUpdated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onReadPolicyRecord: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            if (!_view.selection) return;
            let _form = Ext.create('App.Control.Kivii.Finances.PolicyRecord.List', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                autoScroll: true,
                width: 1200,
                height: 680,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_view.selection.data);
        },
        _onReadDetail: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _view = _controller.getView();
            let _form = Ext.create('App.Control.Kivii.Finances.PolicyDetail.List', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 800,
                height: 480,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_record.data);
        }
    }
});