﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountCreate : RestfulCreate<Account>
    {
        private IDbTransaction _trans;
        public override bool OnPreRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<Account> rtns)
        {
            if (Item.OwnerKvid == Guid.Empty) throw new Exception("未指定账户所有人");
            if (Item.OwnerName.IsNullOrEmpty()) throw new Exception("未指定账户所有人名称");
            if (Item.Currency == CurrencyUnit.Unsupported) throw new Exception("不支持的货币类型！");
            if (Item.Type == AccountType.Unsupported || Item.Type == AccountType.Balance || Item.Type == AccountType.Biz || Item.Type == AccountType.Coupon || Item.Type == AccountType.Pos || Item.Type == AccountType.StoredCard || Item.Type == AccountType.Cash) throw new Exception("不支持的账户类型！");
            if (Item.Amount != 0) throw new Exception("开户时，户内金额只能为零！");

            if (Item.Type == AccountType.Deposit && dbConnection.Exists<Account>(o => o.Name == Item.Name && o.OwnerKvid == Item.OwnerKvid && o.Type == AccountType.Deposit && o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid)) throw new Exception("当前组织已存在此客户的存款账户");
            //if (Item.Type == AccountType.WeChat && dbConnection.Exists<Account>(o => o.OwnerKvid == Item.OwnerKvid && o.Type == AccountType.WeChat && o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid)) throw new Exception("当前组织已存在微信账户!");
            //if (Item.Type == AccountType.AliPay && dbConnection.Exists<Account>(o => o.OwnerKvid == Item.OwnerKvid && o.Type == AccountType.AliPay && o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid)) throw new Exception("当前组织已存在支付宝账户!");
            //if (Item.Type == AccountType.Bank && dbConnection.Exists<Account>(o => o.OwnerKvid == Item.OwnerKvid && o.Type == AccountType.Bank && o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid)) throw new Exception("当前组织已存在银行账户!");
            if (dbConnection is LinqConnection && (dbConnection as LinqConnection).DbTransaction == null)
            {
                _trans = dbConnection.OpenTransaction();
            }
            return base.OnPreRestfulCreate(req, res, dbConnection, rtns);
        }

        public override bool OnPostRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<Account> rtns)
        {
            try
            {
                //新建开户记录
                var accountDetail = new AccountDetail();
                accountDetail.AccountKvid = Item.Kvid;
                accountDetail.BizKvid = Item.Kvid;
                accountDetail.BizType = typeof(Account).FullName;
                accountDetail.BizId = Item.SerialNumber;

                accountDetail.PayerKvid = Item.Kvid;
                accountDetail.PayerName = Item.Name;
                accountDetail.PayerAccountKvid = Item.Kvid;
                accountDetail.PayerAccountName = Item.Name;
                accountDetail.PayerAccountSerialNumber = Item.SerialNumber;

                accountDetail.PayeeKvid = Item.Kvid;
                accountDetail.PayeeName = Item.Name;
                accountDetail.PayeeAccountKvid = Item.Kvid;
                accountDetail.PayeeAccountName = Item.Name;
                accountDetail.PayeeAccountSerialNumber = Item.SerialNumber;

                accountDetail.AmountPayment = 0;
                accountDetail.Amount = 0;

                accountDetail.Summary = "开户";
                accountDetail.Remark = "开户";
                dbConnection.Insert(accountDetail);

                _trans?.Commit();
                return base.OnPostRestfulCreate(req, res, dbConnection, rtns);
            }
            catch (Exception ex)
            {
                _trans?.Rollback();
                throw ex;
            }
            finally
            {
                _trans?.Dispose();
            }
        }
    }

    [Api(Description = "账户查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountQueryEx : RestfulExecution<Account>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Account>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Account>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            var ownerKvids = KiviiContext.CurrentMember.DepartmentKvids.ToList();
            ownerKvids.Add(KiviiContext.CurrentMember.OrganizationKvid);
            ownerKvids.Add(KiviiContext.CurrentMember.Kvid);
            sqlExpress.Where(o => o.Type != AccountType.Balance && Sql.In(o.OwnerKvid, ownerKvids));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountQuery : RestfulQuery<Account>
    {

    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountRead : RestfulRead<Account>
    {
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountUpdate : RestfulUpdate<Account>
    {

    }

    [Api(Description = "获取折扣账户")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountGetDiscount : RestfulExecution<Account>
    {
        public CurrencyUnit Currency { get; set; }

        public string OwnerName { get; set; }

        public Guid OwnerKvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位!");
            
            var rtns = new RestfulReadResponse<Account>();
            rtns.Result = Currency.GetDiscountAccount(OwnerKvid, OwnerName);

            return rtns;
        }
    }

    [Api(Description = "获取部门业务账户")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountGetBizAccount : RestfulExecution<Account>
    {
        public CurrencyUnit Currency { get; set; }

        public string OwnerName { get; set; }

        public Guid OwnerKvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位!");

            var rtns = new RestfulReadResponse<Account>();
            rtns.Result = Currency.GetBizAccount(OwnerKvid, OwnerName);

            return rtns;
        }
    }

    [Api(Description = "获取平衡账户")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountGetBalance : RestfulExecution<Account>
    {
        public CurrencyUnit Currency { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位!");

            var rtns = new RestfulReadResponse<Account>();
            rtns.Result = Currency.GetBalanceAccount();

            return rtns;
        }
    }

    [Api(Description = "获取个人账户")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class AccountGetPersonal : RestfulExecution<Account>
    {
        public CurrencyUnit Currency { get; set; }

        public AccountType AccountType { get; set; }
        public string OwnerName { get; set; }

        public Guid OwnerKvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位!");
            (AccountType != AccountType.Cash && AccountType != AccountType.Pos).ThrowIfTrue("不支持的账户类型!,个人账户仅支持现金和刷卡账户!");
            var rtns = new RestfulReadResponse<Account>();
            rtns.Result = Currency.GetPersonalAccount(AccountType, OwnerKvid, OwnerName);

            return rtns;
        }
    }
}
