﻿using Kivii.Finances.Entities;
using Kivii.Finances.Extensions;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "泛型结算创建")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementCreate : RestfulExecutionGeneric<Settlement>
    {
        public Settlement Item { get; set; }
        public List<Settlement> Items { get; set; }

        public bool AutoGenerateBill { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("请传入要结算的内容!");
            if (Items.IsNullOrEmpty()) Items = new List<Settlement>();
            if (Item != null) Items.Add(Item);
            Items.Exists(o => o.PayerName.IsNullOrEmpty()).ThrowIfTrue("付款单位不能为空!");
            Items.Exists(o => o.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位!");
            Items.Exists(o => o.Details.IsNullOrEmpty()).ThrowIfTrue("结算明细不能为空!");
            var bizKvids = Items.ConvertAll(p => p.BizKvid);
            bizKvids.RemoveAll(o => o == Guid.Empty);
            var bizids = Items.ConvertAll(p => p.BizId).Distinct().ToList();
            bizids.RemoveAll(o => o.IsNullOrEmpty());

            #region 启用缓存,将当前人创建批次数据存入,处理完毕或者接口报错后清除,确保不会重复创建
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = KiviiContext.GetUrnKey($"{this.GetType().FullName}_Request_{KiviiContext.CurrentMember.FullName}_{KiviiContext.CurrentMember.Kvid}");
            var cacheValue = cache.Get<int?>(cacheKey);
            if (cacheValue != null)
            {
                if (cacheValue < 0) throw new Exception($"仍在处理中,请稍后查看结果!");
                else throw new Exception($"当前正在处理中,剩余{cacheValue}条,请稍后查看结果!");
            }
            int? caches = Items.Count;
            //caches.AddRange(Items.ConvertAll(o => o.SerialNumber));
            cache.Set(cacheKey, caches, TimeSpan.FromMinutes(5));
            #endregion

            try
            {

                var connF = KiviiContext.GetOpenedDbConnection<Settlement>();
                //foreach (var bizId in bizids)
                //{
                //    if (connF.Exists<Settlement>(o => o.OffsetKvid == Guid.Empty && o.BizId == bizId)) throw new Exception($"{bizId}重复结算的对象!");
                //}
                if (connF.Exists<Settlement>(o => o.OffsetKvid == Guid.Empty && (Sql.In(o.BizKvid, bizKvids) || Sql.In(o.BizId, bizids)))) throw new Exception("存在重复结算的对象!");
                var connE = KiviiContext.GetOpenedDbConnection<G>();
                //如果泛型类型不是Settlement本身就需要判断
                if (typeof(G) != typeof(Settlement)) connE.Exists<EntitySettlement<G>>(o => o.OffsetKvid == Guid.Empty && Sql.In(o.BizId, bizids)).ThrowIfTrue("存在重复结算的对象!");
                //var allDetails = new List<SettlementDetail>();
                //Items.ForEach(o => allDetails.AddRange(o.Details));
                //var detailBizKvids = allDetails.ConvertAll(p => p.BizKvid);
                //detailBizKvids.RemoveAll(o => o == Guid.Empty);
                //var detailBizids = allDetails.ConvertAll(p => p.BizId);
                //detailBizids.RemoveAll(o => o.IsNullOrEmpty());
                //if (connE.Exists<EntitySettlementDetail<G>>(o => o.OffsetKvid != Guid.Empty && (Sql.In(o.BizKvid, detailBizKvids) || Sql.In(o.BizId, detailBizids)))) throw new Exception("存在重复结算的明细项目!");
                var rtns = new RestfulCreateResponse<Settlement>();
                rtns.Results = new List<Settlement>();

                #region  合并数据库连接,并启用事务
                IDbTransaction tranE = null, tranF = null;
                //if (connE.ConnectionString == connR.ConnectionString)//现在的连接会自动取缓存
                if (connE == connF)
                {
                    //两个连接是同一个，缓存生效,不做任何操作
                }
                else if (connE.ConnectionString == connF.ConnectionString)//连接对象不同，但连接字符串是一样的，因为在本环节中，连接没有被污染，可以关了一个连接
                {
                    connE.Close();
                    connE = connF;
                }
                if ((connE is LinqConnection) && (connE as LinqConnection).Transaction == null)
                {
                    tranE = connE.OpenTransaction();
                }
                if ((connF is LinqConnection) && (connF as LinqConnection).Transaction == null)
                {
                    tranF = connF.OpenTransaction();
                }
                //#region  合并数据库连接,并启用事务
                //IDbTransaction tranE = null, tranF = null;
                //if (connE.ConnectionString == connF.ConnectionString)
                //{
                //    connF.Close();
                //    connF = connE;
                //    tranE = connE.OpenTransaction();
                //}
                //else
                //{
                //    tranE = connE.OpenTransaction();
                //    tranF = connF.OpenTransaction();
                //}
                #endregion

                try
                {
                    using (KiviiContext.Profiler("创建结算"))
                    {
                        foreach (var item in Items)
                        {
                            if (item.Amount != item.Details.Sum(o => o.Amount) || item.AmountPlan != item.Details.Sum(o => o.AmountPlan)) throw new Exception("项目明细总费用和结算主体总费用不一致!");
                            var settlementKvid = Guid.NewGuid();
                            #region 生成财务 Settlement 主结算
                            var settlement = new Settlement();
                            settlement.PopulateWith(item);
                            settlement.Kvid = settlementKvid;
                            if (settlement.OwnerKvid == Guid.Empty)
                            {
                                settlement.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                                settlement.OwnerName = KiviiContext.CurrentMember.DepartmentName;
                            }
                            settlement.BizId = item.BizId;
                            settlement.BizKvid = item.BizKvid;
                            settlement.BizType = typeof(G).FullName;

                            settlement.OperateTime = item.OperateTime == DateTime.MinValue ? DateTime.Now : item.OperateTime;
                            settlement.OperatorName = KiviiContext.CurrentMember.FullName;
                            settlement.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                            connF.Insert(settlement);
                            settlement.RemoveAllOnlyProperties();
                            rtns.Results.Add(settlement);

                            if (typeof(G) != typeof(Settlement))
                            {
                                var entitySettlement = new EntitySettlement<G>();
                                entitySettlement.PopulateInstance(settlement);
                                connE.Insert(entitySettlement);
                            }
                            #endregion

                            #region 回写更新G数据
                            var instance = Activator.CreateInstance(typeof(G)) as IEntityHasSettlement;
                            if (instance != null)
                            {
                                instance.Kvid = item.BizKvid;
                                instance.AmountSettlement = settlement.Amount;
                                instance.AddOnlyProperties(o => o.AmountSettlement);
                                connE.UpdateOnly(instance);
                            }
                            #endregion

                            #region 生成泛型SettlementDetail
                            foreach (var detail in item.Details)
                            {
                                detail.SettlementKvid = settlementKvid;
                                detail.Quantity = detail.Quantity <= 0 ? 1 : detail.Quantity;
                                detail.QuantityUnit = detail.QuantityUnit.IsNullOrEmpty() ? "次" : detail.QuantityUnit;
                                detail.QuantityUnitPrice = detail.Amount / detail.Quantity;
                                detail.Currency = item.Currency;
                                detail.OperateTime = DateTime.Now;
                                detail.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                                detail.OperatorName = KiviiContext.CurrentMember.FullName;
                                var entityDetail = new EntitySettlementDetail<G>();
                                entityDetail.PopulateInstance(detail);
                                connE.Insert(entityDetail);
                            }
                            #endregion

                            var existPolicy = item.Details.FirstOrDefault(o => o.BizType == typeof(Policy).FullName);
                            if (existPolicy != null)
                            {
                                var policyRecord = new PolicyRecord();
                                policyRecord.PolicyKvid = existPolicy.BizKvid;
                                policyRecord.SettlementKvid = settlementKvid;
                                policyRecord.BizId = settlement.BizId;
                                policyRecord.BizKvid = settlement.BizKvid;
                                policyRecord.BizType = settlement.BizType;
                                policyRecord.TargetKvid = settlement.PayerKvid;
                                policyRecord.TargetName = settlement.PayerName;
                                policyRecord.AmountPlan = item.Details.Sum(o => o.AmountPlan);
                                policyRecord.AmountOrigin = item.Details.Where(o => o.BizType != typeof(Policy).FullName).Sum(o => o.Amount);
                                policyRecord.AmountPolicy = Math.Abs(item.Details.Where(o => o.BizType == typeof(Policy).FullName).Sum(o => o.Amount));
                                policyRecord.Amount = item.Details.Sum(o => o.Amount);
                                policyRecord.OperateTime = settlement.OperateTime;
                                policyRecord.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                                policyRecord.OperatorName = KiviiContext.CurrentMember.FullName;
                                policyRecord.Metadata = new Dictionary<string, string>();
                                foreach (var propertyInfo in settlement.GetType().GetProperties())
                                {
                                    if (propertyInfo.Name == "Metadata") continue;
                                    try
                                    {
                                        var value = propertyInfo.GetValue(item);
                                        policyRecord.Metadata[propertyInfo.Name] = value == null ? string.Empty : value.ToString();
                                    }
                                    catch { continue; }
                                }
                                connF.Insert(policyRecord);
                            }
                            caches--;
                            cache.Replace(cacheKey, caches);
                        }
                    }
                    tranE?.Commit();
                    tranF?.Commit();
                    cache.Remove(cacheKey);
                }
                catch (Exception ex)
                {
                    tranE?.Rollback();
                    tranF?.Rollback();
                    cache.Remove(cacheKey);
                    throw ex;
                }

                if(AutoGenerateBill)
                {
                    var bill = new Bill();
                    bill.Summary = "结算后自动生成账单";
                    bill.PayeeKvid = KiviiContext.CurrentMember.OrganizationKvid;
                    bill.PayeeName = KiviiContext.CurrentMember.OrganizationName;
                    bill.Currency = CurrencyUnit.CNY;
                    bill.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                    bill.OperatorName = KiviiContext.CurrentMember.FullName;
                    bill.PayerKvid = rtns.Results[0].PayerKvid;
                    bill.PayerName = rtns.Results[0].PayerName;
                    bill.Amount = rtns.Results.Sum(o => o.Amount);
                    bill.AmountPlan = rtns.Results.Sum(o => o.AmountPlan);

                    var th = KiviiContext.NewThread(() =>
                    {
                        bill.Accept(rtns.Results);
                    });
                    th.Start();
                }

                return rtns;
            }
            catch (Exception ex)
            {
                cache.Remove(cacheKey);
                throw ex;
            }
        }
    }

    [Api(Description = "泛型结算更新")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementUpdate : RestfulExecutionGeneric<Settlement>
    {
        public Settlement Item { get; set; }
        public List<Settlement> Items { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("请传入要结算的内容!");
            if (Items.IsNullOrEmpty()) Items = new List<Settlement>();
            if (Item != null) Items.Add(Item);

            var connF = KiviiContext.GetOpenedDbConnection<Settlement>();
            var connE = KiviiContext.GetOpenedDbConnection<G>();
            var rtns = new RestfulUpdateResponse<Settlement>();
            rtns.Results = new List<Settlement>();

            #region  合并数据库连接,并启用事务
            IDbTransaction tranE = null, tranF = null;
            //if (connE.ConnectionString == connR.ConnectionString)//现在的连接会自动取缓存
            if (connE == connF)
            {
                //两个连接是同一个，缓存生效,不做任何操作
            }
            else if (connE.ConnectionString == connF.ConnectionString)//连接对象不同，但连接字符串是一样的，因为在本环节中，连接没有被污染，可以关了一个连接
            {
                connE.Close();
                connE = connF;
            }
            if ((connE is LinqConnection) && (connE as LinqConnection).Transaction == null)
            {
                tranE = connE.OpenTransaction();
            }
            if ((connF is LinqConnection) && (connF as LinqConnection).Transaction == null)
            {
                tranF = connF.OpenTransaction();
            }
            //#region  合并数据库连接,并启用事务
            //IDbTransaction tranE = null, tranF = null;
            //if (connE.ConnectionString == connF.ConnectionString)
            //{
            //    connF.Close();
            //    connF = connE;
            //    tranE = connE.OpenTransaction();
            //}
            //else
            //{
            //    tranE = connE.OpenTransaction();
            //    tranF = connF.OpenTransaction();
            //}
            #endregion

            try
            {
                foreach (var item in Items)
                {
                    if (item.OnlyProperties.IsNullOrEmpty()) continue;
                    item.OnlyProperties.RemoveAll(o => o != "Remark" && o != "Summary" && o != "Category");
                    if (item.OnlyProperties.IsNullOrEmpty()) continue;
                    connF.UpdateOnly(item);
                    rtns.Results.Add(item);

                    if (typeof(G) != typeof(Settlement))
                    {
                        var entitySettlement = new EntitySettlement<G>();
                        entitySettlement.PopulateInstance(item);
                        connE.UpdateOnly(entitySettlement);
                    }
                }
                tranE?.Commit();
                tranF?.Commit();
            }
            catch (Exception ex)
            {
                tranE?.Rollback();
                tranF?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [Api(Description = "结算读取")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementRead : RestfulExecutionGeneric<Settlement>
    {
        public Guid Kvid { get; set; }

        public bool IncludeDetail { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<EntitySettlement<G>>();
            var rtns = new RestfulReadResponse<EntitySettlement<G>>();
            rtns.Result = new EntitySettlement<G>();
            if (typeof(G) != typeof(Settlement)) rtns.Result = conn.SingleById<EntitySettlement<G>>(Kvid);
            else
            {
                var settlement = conn.SingleById<Settlement>(Kvid);
                if (settlement != null) rtns.Result.PopulateWith(settlement);
            }
            if (IncludeDetail)
            {
                var details = conn.Select<EntitySettlementDetail<G>>(o => o.SettlementKvid == Kvid && o.OffsetKvid == Guid.Empty);
                rtns.Result.Details = new List<SettlementDetail>();
                foreach (var item in details)
                {
                    var detail = new SettlementDetail();
                    detail.PopulateWith(item);
                    rtns.Result.Details.Add(detail);
                }
            }
            return rtns;
        }
    }

    [Api(Description = "结算查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementQuery : RestfulQuery<Settlement>
    {
        public Guid BillKvid { get; set; }

        public override bool OnPreRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Settlement> rtns, ref Expression<Func<Settlement, bool>> where)
        {
            if (BillKvid != Guid.Empty)
            {
                var queryDetails = dbConnection.From<BillDetail>();
                queryDetails.Where(o => o.BillKvid == BillKvid);
                queryDetails.Select(o => o.BizKvid);
                where = o => Sql.In(o.Kvid, queryDetails);
            }
            return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
        }
    }

    [Api(Description = "结算查询Ex")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementQueryEx : RestfulExecution<Settlement>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public bool? IsPayed { get; set; }//是否已经收款

        public bool? IsBilled { get; set; }//是否已经制作过账单

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Settlement>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            if (IsPayed != null)
            {
                if (IsPayed.Value) sqlExpress.And(o => o.AmountPayment == o.Amount || o.Amount == 0);
                else sqlExpress.And(o => o.Amount != 0 && (o.AmountPayment < o.Amount || o.AmountPayment <= 0));
            }
            if (IsBilled != null)
            {
                var queryBillDetails = conn.From<BillDetail>();
                queryBillDetails.Where(o => o.OffsetKvid == Guid.Empty);
                queryBillDetails.Select(o => o.BizKvid);
                if (IsBilled.Value) sqlExpress.And(o => Sql.In(o.Kvid, queryBillDetails));
                else sqlExpress.And(o => !Sql.In(o.Kvid, queryBillDetails));
            }

            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }

    }

    [Api(Description = "结算删除")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementDelete : RestfulExecutionGeneric<Settlement>
    {
        public List<Guid> Kvids { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("参数错误:未传入要删除的结算Kvids");

            var connF = KiviiContext.GetOpenedDbConnection<Settlement>();
            var connE = KiviiContext.GetOpenedDbConnection<EntitySettlement<G>>();

            connF.Exists<Settlement>(o => o.AmountPayment != 0 && Sql.In(o.Kvid, Kvids)).ThrowIfTrue("删除错误:删除的结算中包含已收款的结算!");
            //如果泛型类型不是Settlement本身就需要判断
            if (typeof(G) != typeof(Settlement)) connE.Exists<EntitySettlement<G>>(o => o.AmountPayment != 0 && Sql.In(o.Kvid, Kvids)).ThrowIfTrue("删除错误:删除的结算中包含已收款的结算!");

            var settlements = connF.Select<Settlement>(o => Sql.In(o.Kvid, Kvids));
            //var entitySettlements = connE.Select<EntitySettlement<G>>(o => Sql.In(o.Kvid, Kvids));
            //var entitySettlementDetails = connE.Select<EntitySettlementDetail<G>>(o => Sql.In(o.SettlementKvid, Kvids));
            connF.InitEntityType<BillDetail>();
            connF.InitEntityType<Bill>();
            connF.InitEntityType<PolicyRecord>();
            var billDetails = connF.Select<BillDetail>(o => Sql.In(o.BizKvid, Kvids));
            var offsetBillDetails = new List<BillDetail>();
            foreach (var item in billDetails)
            {
                offsetBillDetails.Add(item.Offset("结算项已删除"));
            }
            var policyRecords = connF.Select<PolicyRecord>(o => Sql.In(o.SettlementKvid, Kvids));

            var queryBillKvids = connF.From<BillDetail>();
            queryBillKvids.SelectDistinct(o => o.BillKvid);
            queryBillKvids.Where(o => Sql.In(o.BizKvid, Kvids));
            //查出所有的Bill Kvid集合
            var billKvids = connF.Column<Guid>(queryBillKvids);

            var rtns = new RestfulDeleteResponse<Settlement>();
            rtns.Results = new List<Guid>();

            var logs = new List<EntityLog<Settlement>>();
            foreach (var item in settlements)
            {

                var log = new EntityLog<Settlement>();
                log.OwnerKvid = item.Kvid;
                log.BizId = item.BizId;
                log.BizKvid = item.Kvid;
                log.Type = "结算作废或删除";
                log.BizType = typeof(Settlement).FullName;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "结算删除";
                log.Remark = $"{item.BizId}结算已经删除.";
                logs.Add(log);
            }
            #region  合并数据库连接,并启用事务
            IDbTransaction tranE = null, tranF = null;
            //if (connE.ConnectionString == connR.ConnectionString)//现在的连接会自动取缓存
            if (connE == connF)
            {
                //两个连接是同一个，缓存生效,不做任何操作
            }
            else if (connE.ConnectionString == connF.ConnectionString)//连接对象不同，但连接字符串是一样的，因为在本环节中，连接没有被污染，可以关了一个连接
            {
                connE.Close();
                connE = connF;
            }
            if ((connE is LinqConnection) && (connE as LinqConnection).Transaction == null)
            {
                tranE = connE.OpenTransaction();
            }
            if ((connF is LinqConnection) && (connF as LinqConnection).Transaction == null)
            {
                tranF = connF.OpenTransaction();
            }
            //#region  合并数据库连接,并启用事务
            //IDbTransaction tranE = null, tranF = null;
            //if (connE.ConnectionString == connF.ConnectionString)
            //{
            //    connF.Close();
            //    connF = connE;
            //    tranE = connE.OpenTransaction();
            //}
            //else
            //{
            //    tranE = connE.OpenTransaction();
            //    tranF = connF.OpenTransaction();
            //}
            #endregion
            try
            {
                //删除Settlement的数据
                var updateSettlements = connF.From<Settlement>();
                updateSettlements = updateSettlements.Update(o => o.Status);
                updateSettlements = updateSettlements.Where(o => Sql.In(o.Kvid, Kvids));
                connF.UpdateOnly<Settlement>(new Settlement { Status = -1 }, updateSettlements);

                //删除泛型Settlement的数据
                if (typeof(G) != typeof(Settlement))
                {
                    var updateEntitySettlements = connE.From<EntitySettlement<G>>();
                    updateEntitySettlements = updateEntitySettlements.Update(o => o.Status);
                    updateEntitySettlements = updateEntitySettlements.Where(o => Sql.In(o.Kvid, Kvids));
                    connE.UpdateOnly<EntitySettlement<G>>(new EntitySettlement<G> { Status = -1 }, updateEntitySettlements);

                    #region 回写更新G数据
                    var instance = Activator.CreateInstance(typeof(G)) as IEntityHasSettlement;
                    if (instance != null)
                    {
                        foreach (var item in settlements)
                        {
                            instance.Kvid = item.BizKvid;
                            instance.AmountSettlement = 0;
                            instance.AddOnlyProperties(o => o.AmountSettlement);
                            connE.UpdateOnly(instance);
                        }
                    }
                    #endregion
                }

                //删除泛型SettlementDetail的数据
                var updateEntitySettlementDetails = connE.From<EntitySettlementDetail<G>>();
                updateEntitySettlementDetails = updateEntitySettlementDetails.Update(o => o.Status);
                updateEntitySettlementDetails = updateEntitySettlementDetails.Where(o => Sql.In(o.SettlementKvid, Kvids));
                connE.UpdateOnly<EntitySettlementDetail<G>>(new EntitySettlementDetail<G> { Status = -1 }, updateEntitySettlementDetails);

                if (!policyRecords.IsNullOrEmpty())
                {
                    foreach (var record in policyRecords)
                    {
                        var offset = record.Offset("结算删除");
                        connF.UpdateOnly(record);
                        connF.Insert(offset);
                    }
                }
                //作废账单中包含当前结算项的明细
                #region 作废账单中包含当前结算项的明细
                foreach (var item in billDetails)
                {
                    connF.UpdateOnly(item);
                }
                foreach (var item in offsetBillDetails)
                {
                    connF.Insert(item);
                }
                if (!billKvids.IsNullOrEmpty())
                {
                    var updateBills = connE.From<Bill>();
                    updateBills = updateBills.Update(o => o.Summary);
                    updateBills = updateBills.Where(o => Sql.In(o.Kvid, billKvids));
                    connE.UpdateOnly<Bill>(new Bill { Summary = "当前账单结算项目可能被删除或作废,请查看详情或重新制作账单!" }, updateBills);
                }
                #endregion
                foreach (var item in logs)
                {
                    connF.Insert(item);
                }
                rtns.Results.AddRange(Kvids);

                tranF?.Commit();
                tranE?.Commit();
            }
            catch (Exception ex)
            {
                tranF?.Rollback();
                tranE?.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    [Api(Description = "泛型结算明细Query")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementDetailGenericQuery : RestfulExecutionGeneric<SettlementDetail>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<EntitySettlementDetail<G>>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<EntitySettlementDetail<G>>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);

            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);

            return rtns;
        }
    }

    [Api(Description = "结算调价")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementModifyAmount : RestfulExecutionGeneric<Settlement>
    {
        public List<Guid> Kvids { get; set; }

        public decimal Amount { get; set; }

        public string Remark { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("参数错误:未传入要调价的结算Kvids");
            (Amount < 0).ThrowIfTrue("调整总价格不能为负值!");
            var connFins = KiviiContext.GetOpenedDbConnection<Settlement>();
            connFins.Exists<Settlement>(o => (o.AmountPayment != 0 || o.OffsetKvid != Guid.Empty) && Sql.In(o.Kvid, Kvids)).ThrowIfTrue("调价错误:存在已支付或待支付或已冲销的结算!");
            var connG = KiviiContext.GetOpenedDbConnection<EntitySettlement<G>>();
            //如果泛型类型不是Settlement本身就需要判断
            if (typeof(G) != typeof(Settlement)) connG.Exists<EntitySettlement<G>>(o => (o.AmountPayment != 0 || o.OffsetKvid != Guid.Empty) && Sql.In(o.Kvid, Kvids)).ThrowIfTrue("调价错误:存在已支付或待支付或已冲销的结算!");

            var querySettlements = connFins.From<Settlement>();
            querySettlements.Where(o => Sql.In(o.Kvid, Kvids));
            querySettlements.OrderBy(o => o.Amount);
            var settlements = connFins.Select(querySettlements);
            settlements.ThrowIfNullOrEmpty("未找到要调价的结算单!");
            (settlements.Count != Kvids.Count).ThrowIfTrue("所选结算单信息不完整!");
            settlements.Exists(o => o.AmountPayment != 0 || o.OffsetKvid != Guid.Empty).ThrowIfTrue("调价错误:存在已支付或待支付或已冲销的结算!");
            var amountSum = settlements.Sum(o => o.Amount);
            var rates = new Dictionary<Guid, decimal>();
            foreach (var item in settlements)
            {
                var rate = Math.Round(item.Amount / amountSum, 2);
                rates.Add(item.Kvid, rate);
            }
            if (rates.ContainsKey(Guid.Empty)) throw new Exception("存在Kvid为空值!");
            //差值
            var deviation = 1 - rates.Sum(o => o.Value);
            if (deviation > 0)
            {
                var first = rates.FirstOrDefault();
                rates[first.Key] += deviation;
            }
            else if (deviation < 0)
            {
                foreach (var rate in rates)
                {
                    if (rate.Value + deviation <= 0) continue;
                    //{
                    //    rates[rate.Key] += deviation * rate.Value;
                    //    deviation -= deviation * rate.Value;
                    //}
                    else rates[rate.Key] += deviation;
                }
            }
            if (rates.Sum(o => o.Value) != 1) throw new Exception($"总比和:{rates.Sum(o => o.Value)},比例错误!");

            var amountDeviation = Amount - amountSum;
            foreach (var item in settlements)
            {
                decimal rate = 0;
                if (!rates.TryGetValue(item.Kvid, out rate)) throw new Exception($"{item.BizId},{item.Kvid}");
                var amount = rate * amountDeviation;
                item.Amount += amount;
                item.AddOnlyProperties(o => o.Amount);
            }
            if (settlements.Sum(o => o.Amount) != Amount) throw new Exception($"批量调价总额出错:{settlements.Sum(o => o.Amount)}");

            var rtns = new RestfulQueryResponse<Settlement>();
            rtns.Results = new List<Settlement>();

            #region  合并数据库连接,并启用事务
            IDbTransaction tranG = null, tranFins = null;
            //if (connE.ConnectionString == connR.ConnectionString)//现在的连接会自动取缓存
            if (connG == connFins)
            {
                //两个连接是同一个，缓存生效,不做任何操作
            }
            else if (connG.ConnectionString == connFins.ConnectionString)//连接对象不同，但连接字符串是一样的，因为在本环节中，连接没有被污染，可以关了一个连接
            {
                connG.Close();
                connG = connFins;
            }
            if ((connG is LinqConnection) && (connG as LinqConnection).Transaction == null)
            {
                tranG = connG.OpenTransaction();
            }
            if ((connFins is LinqConnection) && (connFins as LinqConnection).Transaction == null)
            {
                tranFins = connFins.OpenTransaction();
            }
            //#region  合并数据库连接,并启用事务
            //IDbTransaction tranG = null, tranFins = null;
            //if (connG.ConnectionString == connFins.ConnectionString)
            //{
            //    connFins.Close();
            //    connFins = connG;
            //    tranG = connG.OpenTransaction();
            //}
            //else
            //{
            //    tranG = connG.OpenTransaction();
            //    tranFins = connFins.OpenTransaction();
            //}
            #endregion

            try
            {
                foreach (var item in settlements)
                {
                    connFins.UpdateOnly(item);
                    rtns.Results.Add(item);
                    decimal rate = 0;
                    if (!rates.TryGetValue(item.Kvid, out rate)) throw new Exception($"{item.BizId},{item.Kvid}");
                    var amount = rate * amountDeviation;

                    //如果泛型类型不是Settlement本身就需要判断
                    if (typeof(G) != typeof(Settlement))
                    {
                        var entitySettlement = new EntitySettlement<G>();
                        entitySettlement.PopulateWith(item);
                        entitySettlement.Kvid = item.Kvid;
                        entitySettlement.Amount = item.Amount;
                        entitySettlement.AddOnlyProperties(o => o.Amount);
                        connG.UpdateOnly(entitySettlement);
                    }

                    #region 新增结算明细
                    var detail = new EntitySettlementDetail<G>();
                    detail.SettlementKvid = item.Kvid;
                    detail.OperateTime = DateTime.Now;
                    detail.OperatorName = KiviiContext.CurrentMember.FullName;
                    detail.OperatorKvid = KiviiContext.CurrentMember.Kvid;

                    detail.BizId = item.BizId;
                    detail.BizKvid = item.BizKvid;
                    detail.BizType = item.BizType;
                    detail.Amount = amount;
                    detail.AmountPlan = 0;
                    detail.GoodsFullName = "调价";
                    detail.GoodsUnit = "次";
                    detail.Quantity = 1;
                    detail.QuantityUnitPrice = detail.Amount / detail.Quantity;
                    detail.QuantityPlan = detail.Quantity;
                    detail.QuantityUnitPricePlan = detail.AmountPlan / detail.QuantityPlan;
                    detail.QuantityUnit = "次";
                    detail.Type = "Service";
                    detail.Currency = item.Currency;
                    detail.Remark = $"业务调价原因:{Remark},费用变更:{amount}元,操作人:{KiviiContext.CurrentMember.FullName}";
                    connG.Insert(detail);
                    #endregion

                    #region 回写更新G数据
                    var instance = Activator.CreateInstance(typeof(G)) as IEntityHasSettlement;
                    if (instance != null)
                    {
                        instance.Kvid = item.BizKvid;
                        instance.AmountSettlement = item.Amount;
                        instance.AddOnlyProperties(o => o.AmountSettlement);
                        connG.UpdateOnly(instance);
                    }
                    #endregion
                }

                tranFins?.Commit();
                tranG?.Commit();
            }
            catch (Exception ex)
            {
                tranFins?.Rollback();
                tranG?.Rollback();
                throw ex;
            }
            rtns.Total = rtns.Results.Count();
            return rtns;
        }
    }

    [Api(Description = "结算业务查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementQueryBiz : RestfulExecution<Settlement>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public Guid PaymentKvid { get; set; }
        [ApiMember(Description ="付款Kvid集合")]//收款登记的Payment(PaymentType.DynamicPay),通过当前Payment可以查到对应收款登记的是哪些结算项
        public List<Guid> PaymentKvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (PaymentKvid == Guid.Empty && PaymentKvids.IsNullOrEmpty()).ThrowIfTrue("请传入PaymentKvid");
            if (PaymentKvids.IsNullOrEmpty()) PaymentKvids = new List<Guid>();
            if (PaymentKvid != Guid.Empty) PaymentKvids.Add(PaymentKvid);

            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Settlement>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);

            var queryPays = conn.From<Pay>();
            queryPays.Where(o => o.FromType == PayType.Payment && o.BizType == typeof(Settlement).FullName && o.OffsetKvid == Guid.Empty && Sql.In(o.FromKvid, PaymentKvids));
            queryPays.Select(o => o.BizKvid);

            sqlExpress.Where(o => Sql.In(o.Kvid, queryPays));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }
}